package jp.jsworld.SpModeMailWatcher;

import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import android.accessibilityservice.AccessibilityService;
import android.accessibilityservice.AccessibilityServiceInfo;
import android.app.Notification;
import android.app.NotificationManager;
import android.content.Intent;
import android.media.Ringtone;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Vibrator;
import android.util.Log;
import android.view.accessibility.AccessibilityEvent;

//import android.widget.Toast;

/**
 * SPモードメールを監視するサービスです。<br>
 * <ol>
 * <li>AccessibilityService クラスを継承して通知ステータスの変更イベントを監視します。
 * <li>ユーザ補助でサービスの実行を許可する必要があります。
 * </ol>
 *
 * @version 1.2.0
 * @author jun shiozaki
 *
 *         【ver. 1.2.4 2012/06/23】<br>
 *         ・LEDの点滅が停止しないバグを修正<br>
 *         新着メールの直後に未受信メールを検出しても doCall を呼ばない。 ・鳴動時間を、着信音再生時間とバイブレーター時間に分ける。
 *
 *         【ver. 1.2.3 2012/06/15】<br>
 *         ・toast を表示しないように修正<br>
 *
 *         【ver. 1.2.2 2012/06/06】<br>
 *         ・未受信メールの直後に着信メールを受信した場合にSPモードメールの監視ができなくなるバグを修正<br>
 *
 *         【ver. 1.2.1 2012/06/05】<br>
 *         ・未受信メールの直後に着信メールを受信した場合にLEDの発光が消えないバグを修正<br>
 *
 *         【ver. 1.2.0 2012/06/03】<br>
 *         ・通知をLEDの発光で知らせる処理を追加<br>
 *
 *         【ver. 1.1.0 2012/05/03】<br>
 *         ・使用するメーラーによって未受信メールのテキストを変更するように修正<br>
 */
public class SpModeMailWatchService extends AccessibilityService {
    private static final int NOTIFICATION_TIMEOUT_MILLIS = 1000;
    private static final int NOTIFICATION_INTERVAL = 20000;
    private static final String LOG_TAG = "SpModeMailWatcher";

    //private static final boolean DEBUG_FLG = true;
    private static final boolean DEBUG_FLG = false;

    // 監視タイマー
    private Timer watchTimer;
    private Date noticeTime = new Date();
    private boolean blinkLed = false;

    @Override
    public void onCreate() {
        super.onCreate();
    }

    /**
     * サービスへ接続時の処理を行います。<br>
     * <ol>
     * <li>AccessibilityService の設定を行います。
     * </ol>
     */
    @Override
    public void onServiceConnected() {

        // AccessibilityService の設定を行う。
        // 通知ステータスの変更時のみイベントを発生させる。
        AccessibilityServiceInfo info = new AccessibilityServiceInfo();
        info.eventTypes = AccessibilityEvent.TYPE_NOTIFICATION_STATE_CHANGED;
        info.feedbackType = AccessibilityServiceInfo.FEEDBACK_GENERIC;
        info.notificationTimeout = NOTIFICATION_TIMEOUT_MILLIS;
        info.flags = AccessibilityServiceInfo.DEFAULT;
        setServiceInfo(info);
    }

    @Override
    public void onStart(Intent intent, int startId) {
        super.onStart(intent, startId);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }

    @Override
    public void onAccessibilityEvent(AccessibilityEvent event) {
        if (DEBUG_FLG)
            Log.i(LOG_TAG, "********* onAccessibilityEvent *********");

        // イベントタイプを取得します。
        int eventType = event.getEventType();

        // 念のため TYPE_NOTIFICATION_STATE_CHANGED 以外は無視します。
        if (eventType != AccessibilityEvent.TYPE_NOTIFICATION_STATE_CHANGED)
            return;

        // 通知のテキストを取得します。
        String eventText = "";
        try {
            eventText = (String) event.getText().get(0);
        } catch (Exception e) {
            eventText = "";
        }

        if (DEBUG_FLG)
            Log.i(LOG_TAG, "eventText:" + eventText);

        // テキストが無効の場合は終了する。
        if (eventText == null || eventText.equals("")) {
            return;
        }

        SettingReader sr = new SettingReader(this);
        if (eventText.equals(sr.getUnReceivedMailText())) {
            long waitTime = sr.getWaittime() * 1000;

            // 通知受信時間
            Long span = new Date().getTime() - noticeTime.getTime();
            if (DEBUG_FLG)
                Log.i(LOG_TAG, ">> Notification Span:" + span.toString());

            // 通知が20秒以内に連続して発生した場合はキャンセルする。
            if (span < NOTIFICATION_INTERVAL) {
                if (DEBUG_FLG)
                    Log.i(LOG_TAG, ">> Event Cancel");
                return;
            }

            // 受信時間設定
            noticeTime = new Date();

            if (waitTime <= 0) {
                // 遅延時間が設定されていない場合はタイマーを使用せずにすぐに鳴動します。
                doCall();
            } else {
                // Toast.makeText(this, "未受信メール通知を受信しました",
                // Toast.LENGTH_LONG).show();

                // すでにタイマーがセットされてい場合はキャンセルする。
                if (this.watchTimer != null) {
                    this.watchTimer.cancel();
                }

                // タイマーにジョブを設定する。
                this.watchTimer = new Timer();
                TimerTask timerTask = new TimerTask() {
                    public void run() {
                        doCall();
                    }
                };

                // タイマーを開始して一定時間待機する。
                watchTimer.schedule(timerTask, waitTime);

                // 処理終了
                return;
            }
        }

        // 新着メールのチェックを行います。
        Pattern p = Pattern.compile("^新着メール");
        Matcher m = p.matcher(eventText);
        if (m.find()) {
            // 受信時間設定
            noticeTime = new Date();

            // 新着メールを受信したらタイマーを停止する。
            // Toast.makeText(this, "新着メール通知を受信しました", Toast.LENGTH_LONG).show();
            if (this.watchTimer != null) {
                this.watchTimer.cancel();
            }

            // LED が発光している場合は停止する。
            if (blinkLed) {
                if (DEBUG_FLG)
                    Log.i(LOG_TAG, ">> Stop blink Lights:" + sr.getLedColor());

                // 通知をキャンセルします。
                NotificationManager notificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
                Notification notification = new Notification();
                notificationManager.notify(R.string.app_name, notification);
                blinkLed = false;
            }

            // 処理終了
            return;
        }
    }

    private boolean isCalling = false;
    private Date stTime = null;
    private Timer callTimer = null;
    private Ringtone ringTone = null;
    private Vibrator vibrator = null;

    private boolean soundFlag = true;
    private boolean vibrateFlag = true;
    private boolean ledFlag = true;

    private long soundPlayTime = 0;
    private long vibrateTime = 0;

    /**
     * 未受信メールの通知を行います。
     */
    public void doCall() {

        // すでに呼び出し中の場合は終了する。
        if (isCalling)
            return;

        try {
            if (DEBUG_FLG)
                Log.i(LOG_TAG, "********* doCall *********");
            SettingReader sr = new SettingReader(this);
            String soundName = sr.getSoundName();
            soundPlayTime = sr.getNoticeSoundPlayTime() * 1000;
            vibrateTime = sr.getNoticeVibrateTime() * 1000;

            soundFlag = sr.getNoticeSoundFlag();
            vibrateFlag = sr.getNoticeVibrateFlag();
            ledFlag = sr.getNoticeLedFlag();

            // 通知 LED 発光
            if (ledFlag) {
                if (DEBUG_FLG)
                    Log.i(LOG_TAG, ">> Start blink Lights:" + sr.getLedColor());

                // 通知バーには表示しません。
                NotificationManager notificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
                Notification notification = new Notification();
                notification.flags |= Notification.FLAG_SHOW_LIGHTS;
                notification.ledOnMS = 500;
                notification.ledOffMS = 1500;
                notification.ledARGB = sr.getLedColorRGB(sr.getLedColor());
                notificationManager.notify(R.string.app_name, notification);
                blinkLed = true;
            }

            // 鳴動音の設定
            ringTone = null;
            if (soundFlag) {
                // ※サイレントに設定された場合は null になるので注意。
                if (soundName != null) {
                    Uri uri = Uri.parse(soundName);
                    if (DEBUG_FLG)
                        Log.i(LOG_TAG, "sound URI   ：" + uri.toString());

                    ringTone = RingtoneManager.getRingtone(
                            getApplicationContext(), uri);
                    ringTone.play();
                }
            }

            // バイブレーターの設定
            vibrator = null;
            if (vibrateFlag) {
                if (DEBUG_FLG)
                    Log.i(LOG_TAG, ">> Vibrator:" + sr.getVibratePattern());

                vibrator = sr.getVibrator(sr.getVibratePattern(), 0);
            }

            // 鳴動タイマー設定
            if (DEBUG_FLG)
                Log.i(LOG_TAG, ">> Start Timer");

            stTime = new Date();
            callTimer = new Timer();
            callTimer.schedule(new TimerTask() {
                public void run() {
                    boolean stopSound = false;
                    boolean stopVibrator = false;

                    // 鳴動時間チェック
                    Long span = new Date().getTime() - stTime.getTime();

                    // 着信音再生時間のチェック
                    if (span < soundPlayTime) {
                        // 鳴動音の連続再生
                        if (soundFlag && (ringTone != null)) {
                            if (!ringTone.isPlaying()) {
                                ringTone.play();
                            }
                        }
                    } else {
                        stopSound = true;

                        // 鳴動音の停止
                        if (soundFlag && (ringTone != null)) {
                            if (DEBUG_FLG)
                                Log.i(LOG_TAG, ">> Stop Sound");

                            if (ringTone.isPlaying()) {
                                ringTone.stop();
                                ringTone = null;
                            }
                        }
                    }

                    // バイブレーター時間のチェック
                    if (span < vibrateTime) {

                    } else {
                        stopVibrator = true;
                        if (DEBUG_FLG)
                            Log.i(LOG_TAG, ">> Stop Vibrator");

                        // バイブレーターの停止
                        if (vibrateFlag && (vibrator != null)) {
                            vibrator.cancel();
                            vibrator = null;
                        }
                    }

                    if (stopSound & stopVibrator) {
                        // タイマー停止
                        callTimer.cancel();

                        if (DEBUG_FLG)
                            Log.i(LOG_TAG, ">> Stop Timer");
                    }
                }
            }, 500, 500);

        } catch (Exception e) {
            e.printStackTrace();
        }

        isCalling = false;
    }

    @Override
    public void onInterrupt() {
    }
}
